/*
 * $Id: IteratorEnumeration.java 358692 2005-12-23 03:37:26Z niallp $
 * $Revision: 358692 $
 * $Date: 2005-12-22 22:37:26 -0500 (Thu, 22 Dec 2005) $
 *
 * ====================================================================
 *
 *  Copyright 2005 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.apache.commons.resources.util;

import java.util.Enumeration;
import java.util.Iterator;

/**
 * Two way Adaptor class for java.util.Iterator and java.util.Enumeration.
 */
public class IteratorEnumeration implements Iterator, Enumeration {
    
    private Iterator iterator;
    private Enumeration enumeration;

    // -------------------------- Constructors

    /**
     * Construct an instance which wraps an Iterator.
     *
     * @param iterator The <code>Iterator</code> to wrap.
     */
    public IteratorEnumeration(Iterator iterator) {
        this.iterator = iterator;
    }

    /**
     * Construct an instance which wraps an Enumeration.
     *
     * @param enumeration The <code>Enumeration</code> to wrap.
     */
    public IteratorEnumeration(Enumeration enumeration) {
        this.enumeration = enumeration;
    }

    // -------------------------- Iterator Methods

    /**
     * <code>remove()<code> is not supported.
     * @throws UnsupportedOperationException always.
     */
    public void remove() {
        throw new UnsupportedOperationException();
    }

    /**
     * Indicates whether the wrapped
     * <code>Iterator/Enumeration</code> has more elements.
     *
     * @return <code>true</code> if the wrapped
     * <code>Iterator/Enumeration</code> has more elements.
     */
    public boolean hasNext() {
        if (iterator == null) {
            return enumeration.hasMoreElements();
        } else {
            return iterator.hasNext();
        }
    }

    /**
     * Returns the next element in the wrapped
     * <code>Iterator/Enumeration</code>.
     *
     * @return the next element.
     */
    public Object next() {
        if (iterator == null) {
            return enumeration.nextElement();
        } else {
            return iterator.next();
        }
    }

    // -------------------------- Enumeration Methods

    /**
     * Indicates whether the wrapped
     * <code>Iterator/Enumeration</code> has more elements.
     *
     * @return <code>true</code> if the wrapped
     * <code>Iterator/Enumeration</code> has more elements.
     */
    public boolean hasMoreElements() {
        return hasNext();
    }

    /**
     * Returns the next element in the wrapped
     * <code>Iterator/Enumeration</code>.
     *
     * @return the next element.
     */
    public Object nextElement() {
        return next();
    }

}
